/**
 * 企业相关逻辑
 * 作者: 李欣鸣
 * 包括新注册企业，迁入企业
 * 
 */

import { ERRORENUM } from "../../config/errorEnum";
import { createEnterprise, findEnterpriseByUscc } from "../../data/enterprise/enterprise";
import { BizError } from "../../util/bizError";
import { getTaskId } from "../../util/tools"



/**
 * 新注册企业
 * @param uscc 孵化器统一信用代码
 * @param param 表单参数
 */
export async function registerEnterprise(uscc:string, param) {
    const TaskId = getTaskId(uscc);
    const TypeConf = {
        name:"String", //企业名称
        uscc:"String", //统一信用代码
        industry:"[Number]",//领域
        logonTime:"Number",//注册时间
        firstIncubationTime:"Number",//首次入孵时间
        isNaturalPersonHolding:"Boolean",//是否自然人控股企业
        logonAdd:"String",//注册地址
        operatingAdd:"String",//经营地址
        leasedArea:"Number",//租赁面积（平方米）
    };
    checkParamater(param, TypeConf);
    let dataBaseInfo = await findEnterpriseByUscc(param.uscc);
    
    if ( dataBaseInfo&& dataBaseInfo.uscc)  throw new BizError(ERRORENUM.该企业已存在, `${param.uscc}已经在库中存在`);

    await createEnterprise(uscc, TaskId, param);

    return {isSuccess:true};
}

/**
 * 新注册企业表单修改
 * @param uscc 孵化器统一信用代码
 * @param param 表单参数
 */
export async function updateRegisterEnterprise(uscc:string, param) {
    const TypeConf = {
        name:"String", //企业名称
        uscc:"String", //统一信用代码
        industry:"[Number]",//领域
        logonTime:"Number",//注册时间
        firstIncubationTime:"Number",//首次入孵时间
        isNaturalPersonHolding:"Boolean",//是否自然人控股企业
        logonAdd:"String",//注册地址
        operatingAdd:"String",//经营地址
        leasedArea:"Number",//租赁面积（平方米）
    };
    checkParamater(param, TypeConf);
    let dataBaseInfo = await findEnterpriseByUscc(param.uscc);
    
    if (!dataBaseInfo || !dataBaseInfo.uscc) throw new BizError(ERRORENUM.未找到数据, `库中不存在${param.uscc}这个企业`);


    let changeList = checkChange(param, dataBaseInfo);
    if ( !changeList.length ) throw new BizError(ERRORENUM.数据无更新, `${param.uscc}数据无更新`);

    changeList.forEach(key => {
        dataBaseInfo[key] = param[key];
    });

    await dataBaseInfo.save();

    return {isSuccess:true};
}




/**
 * 迁入企业登记
 * @param uscc 
 * @param param 
 */
export async function moveInEnterprise(uscc:string, param) {
    const TaskId = getTaskId(uscc);
    const TypeConf = {
        name:"String", //企业名称
        uscc:"String", //统一信用代码
        industry:"[Number]",//领域
        logonTime:"Number",//注册时间
        firstIncubationTime:"Number",//首次入孵时间
        timeOfImmigration:"Number",//迁入时间
        isNaturalPersonHolding:"Boolean",//是否自然人控股企业
        oldLogonAdd:"String",//迁入前注册地址
        logonAdd:"String",//注册地址
        operatingAdd:"String",//经营地址
        leasedArea:"Number",//租赁面积（平方米）
    };
    checkParamater(param, TypeConf);
    let dataBaseInfo = await findEnterpriseByUscc(param.uscc);
    
    if ( dataBaseInfo&& dataBaseInfo.uscc)  throw new BizError(ERRORENUM.该企业已存在, `${param.uscc}已经在库中存在`);

    await createEnterprise(uscc, TaskId, param);

    return {isSuccess:true};
}



/**
 * 修改迁入企业登记信息
 * @param uscc 
 * @param param 
 */
export async function updateMoveInEnterprise(uscc:string, param) {
    const TypeConf = {
        name:"String", //企业名称
        uscc:"String", //统一信用代码
        industry:"[Number]",//领域
        logonTime:"Number",//注册时间
        firstIncubationTime:"Number",//首次入孵时间
        timeOfImmigration:"Number",//迁入时间
        isNaturalPersonHolding:"Boolean",//是否自然人控股企业
        oldLogonAdd:"String",//迁入前注册地址
        logonAdd:"String",//注册地址
        operatingAdd:"String",//经营地址
        leasedArea:"Number",//租赁面积（平方米）
    };
    checkParamater(param, TypeConf);
    let dataBaseInfo = await findEnterpriseByUscc(param.uscc);
    
    if (!dataBaseInfo || !dataBaseInfo.uscc) throw new BizError(ERRORENUM.未找到数据, `库中不存在${param.uscc}这个企业`);


    let changeList = checkChange(param, dataBaseInfo);
    if ( !changeList.length ) throw new BizError(ERRORENUM.数据无更新, `${param.uscc}数据无更新`);

    changeList.forEach(key => {
        dataBaseInfo[key] = param[key];
    });

    await dataBaseInfo.save();

    return {isSuccess:true};
    
}


/**
 * 校验参数
 * @param param 
 */
function checkParamater(param, keyTypeConf) {
    for (let key in keyTypeConf ) {
        if ( !param[key] ) throw new BizError(ERRORENUM.表单校验失败, '孵化器基础数据更新', `缺失${key}字段`);
        let type = typeof param[key];
        switch(keyTypeConf[key]) {
            case 'Number':
                if ( type != 'number' ) throw new BizError(ERRORENUM.表单校验失败, '孵化器基础数据更新', `${key}应是number型 而不是${type}`);
            break;
            case 'String':
                if ( type != 'string' ) throw new BizError(ERRORENUM.表单校验失败, '孵化器基础数据更新', `${key}应是string型 而不是${type}`);
            break;
            case 'Boolean':
                if ( type != 'boolean' ) throw new BizError(ERRORENUM.表单校验失败, '孵化器基础数据更新', `${key}应是boolean型 而不是${type}`); 
            break;
            case '[Number]':
                if ( !Array.isArray(param[key]) ) throw new BizError(ERRORENUM.表单校验失败, '孵化器基础数据更新', `${key}应是数组型 而不是${type}`);
                for (let i =0; i <  param[key].length; i++) {
                    let item = param[key][i];
                    if ( typeof item != 'number' ) {
                        throw new BizError(ERRORENUM.表单校验失败, '孵化器基础数据更新', `${key}应是number型数组其中下标${i}是${typeof item}`);
                    }
                }
            break;
        }
    }
    for (let key in param) {
        if (!keyTypeConf[key]) throw new BizError(ERRORENUM.表单校验失败, '孵化器基础数据更新', `多余${key}字段`);
    }
}

function checkChange(newObj, oldObj) {
    let changeKeyList = [];
    for (let newKey in newObj) {
        if (`${newObj[newKey]}` != `${oldObj[newKey]}`) changeKeyList.push(newKey);
    }
    return changeKeyList;
}
