/**
 * 企业融资相关逻辑
 * 作者: 李欣鸣
 * 包括新融资信息的增删改查
 * 
 */

import { ERRORENUM } from "../../config/errorEnum";
import { createFinancing, findFinancingInfoByTaskIdAndSucc } from "../../data/enterprise/financing";
import { BizError } from "../../util/bizError";
import { getTaskId } from "../../util/tools";


export async function createFinancingInfo(uscc, param) {
    const TaskId = getTaskId(uscc);
    const TypeConf = {
        uscc:"String", //统一信用代码 冗余字段
        name:"String",//企业名称
        taskId:"String", //任务id
        logonAdd:"String",//注册地址
        operatingAdd:"String",//经营地址
        financingTime:"Number",//获得融资时间
        financingAmount:"Number",//融资金额(万元)
        investmentInstitutionsName:"String",//投资机构名称
        timeToObtainInvestment:"Number",//获得投资时间
        fuHuaQiInvestment:"Boolean", //孵化器是否投资
        fuHuaQiInvestmentAmount:"Number",//孵化器投资金额(万元)
        fuHuaQiInvestmentStyle:"Number",//孵化器投资方式
        draftLock:"Boolean", //草稿锁，true为提交之后，false为草稿
        createTime:"Number",//录入时间
    };
    checkParamater(param, TypeConf);

    let dataBaseInfo = await findFinancingInfoByTaskIdAndSucc(TaskId, param.uscc);
    
    if ( dataBaseInfo&& dataBaseInfo.uscc)  throw new BizError(ERRORENUM.该企业已存在, `${param.uscc}的融资信息已经存在`);

    await createFinancing(uscc, TaskId, param);

    return {isSuccess:true};
}

export async function updateFinancingInfo(uscc, param) {
    const TaskId = getTaskId(uscc);
    const TypeConf = {
        uscc:"String", //统一信用代码 冗余字段
        name:"String",//企业名称
        taskId:"String", //任务id
        logonAdd:"String",//注册地址
        operatingAdd:"String",//经营地址
        financingTime:"Number",//获得融资时间
        financingAmount:"Number",//融资金额(万元)
        investmentInstitutionsName:"String",//投资机构名称
        timeToObtainInvestment:"Number",//获得投资时间
        fuHuaQiInvestment:"Boolean", //孵化器是否投资
        fuHuaQiInvestmentAmount:"Number",//孵化器投资金额(万元)
        fuHuaQiInvestmentStyle:"Number",//孵化器投资方式
        draftLock:"Boolean", //草稿锁，true为提交之后，false为草稿
        createTime:"Number",//录入时间
    };
    checkParamater(param, TypeConf);
    let dataBaseInfo = await findFinancingInfoByTaskIdAndSucc(TaskId, param.uscc);
    
    if (!dataBaseInfo || !dataBaseInfo.uscc) throw new BizError(ERRORENUM.未找到数据, `库中不存在${TaskId}的任务`);


    let changeList = checkChange(param, dataBaseInfo);
    if ( !changeList.length ) throw new BizError(ERRORENUM.数据无更新, `${param.uscc}数据无更新`);

    changeList.forEach(key => {
        dataBaseInfo[key] = param[key];
    });

    await dataBaseInfo.save();

    return {isSuccess:true};
}


/**
 * 校验参数
 * @param param 
 */
function checkParamater(param, keyTypeConf) {
    for (let key in keyTypeConf ) {
        if ( !param[key] ) throw new BizError(ERRORENUM.表单校验失败, '孵化器基础数据更新', `缺失${key}字段`);
        let type = typeof param[key];
        switch(keyTypeConf[key]) {
            case 'Number':
                if ( type != 'number' ) throw new BizError(ERRORENUM.表单校验失败, '孵化器基础数据更新', `${key}应是number型 而不是${type}`);
            break;
            case 'String':
                if ( type != 'string' ) throw new BizError(ERRORENUM.表单校验失败, '孵化器基础数据更新', `${key}应是string型 而不是${type}`);
            break;
            case 'Boolean':
                if ( type != 'boolean' ) throw new BizError(ERRORENUM.表单校验失败, '孵化器基础数据更新', `${key}应是boolean型 而不是${type}`); 
            break;
            case '[Number]':
                if ( !Array.isArray(param[key]) ) throw new BizError(ERRORENUM.表单校验失败, '孵化器基础数据更新', `${key}应是数组型 而不是${type}`);
                for (let i =0; i <  param[key].length; i++) {
                    let item = param[key][i];
                    if ( typeof item != 'number' ) {
                        throw new BizError(ERRORENUM.表单校验失败, '孵化器基础数据更新', `${key}应是number型数组其中下标${i}是${typeof item}`);
                    }
                }
            break;
        }
    }
    for (let key in param) {
        if (!keyTypeConf[key]) throw new BizError(ERRORENUM.表单校验失败, '孵化器基础数据更新', `多余${key}字段`);
    }
}

function checkChange(newObj, oldObj) {
    let changeKeyList = [];
    for (let newKey in newObj) {
        if (`${newObj[newKey]}` != `${oldObj[newKey]}`) changeKeyList.push(newKey);
    }
    return changeKeyList;
}
