/**
 * 孵化器基础数据相关逻辑
 * 作者: 李某
 * 孵化器相关逻辑 包括孵化器的增删改查
 * 
 */

import { ERRORENUM } from "../../config/errorEnum";
import { findFuHuaQiByUSCC } from "../../data/fuHuaQi/fuhuaqi";
import { BizError } from "../../util/bizError";




/**
 * 更新孵化器基础数据
 * @param uscc 孵化器统一信用代码
 */
/**
 * name:string, virtualEnterpriseNum:number, logonTime:number, incubatedAcreage:number, acreageTotal:number,
    acreagePersonalUse:number, lv:number, identificationTime:number, industry, institutionalNature:number, liaison:string, liaisonPhone:string, 
    personInCharge:string, personInChargePhone:string, personInChargeAdd:string, siteAcreage:number, leasePrice:number
 */
export async function updateFuHuaQiBaseData(uscc:string, param) {

    let baseDataInfo = await findFuHuaQiByUSCC(uscc);
    /**校验参数 */
    checkParamater(param);
    let changeList = checkChange(param, baseDataInfo);

    for (let i = 0; i < changeList.length; i++) {
        let key = changeList[i];
        baseDataInfo[key] = param[key];
    }

    await baseDataInfo.save();

    return {isSuccess:true};
}

/**
 * 校验参数
 * @param param 
 */
function checkParamater(param) {
    let keyTypeConf = {
        name:"String",
        virtualEnterpriseNum:"Number",
        logonTime:"Number",
        incubatedAcreage:"Number",
        acreageTotal:"Number",
        acreagePersonalUse:"Number",
        lv:"Number",
        identificationTime:"Number",
        industry:"[Number]",
        institutionalNature:"Number",
        liaison:"String",
        liaisonPhone:"String",
        personInCharge:"String",
        personInChargePhone:"String",
        personInChargeAdd:"String",
        siteAcreage:"Number",
        leasePrice:"Number"
    };

    for (let key in keyTypeConf ) {
        if ( !param[key] ) throw new BizError(ERRORENUM.表单校验失败, '孵化器基础数据更新', `缺失${key}字段`);
        let type = typeof param[key];
        switch(keyTypeConf[key]) {
            case 'Number':
                if ( type != 'number' ) throw new BizError(ERRORENUM.表单校验失败, '孵化器基础数据更新', `${key}应是number型 而不是${type}`);
            break;
            case 'String':
                if ( type != 'string' ) throw new BizError(ERRORENUM.表单校验失败, '孵化器基础数据更新', `${key}应是number型 而不是${type}`);
            break;
            case '[Number]':
                if ( !Array.isArray(param[key]) ) throw new BizError(ERRORENUM.表单校验失败, '孵化器基础数据更新', `${key}应是数组型 而不是${type}`);
                for (let i =0; i <  param[key].length; i++) {
                    let item = param[key][i];
                    if ( typeof item != 'number' ) {
                        throw new BizError(ERRORENUM.表单校验失败, '孵化器基础数据更新', `${key}应是number型数组其中下标${i}是${typeof item}`);
                    }
                }
            break;
        }
    }
    for (let key in param) {
        if (!keyTypeConf[key]) throw new BizError(ERRORENUM.表单校验失败, '孵化器基础数据更新', `多余${key}字段`);
    }
}

function checkChange(newObj, oldObj) {
    let changeKeyList = [];
    for (let newKey in newObj) {
        if (`${newObj[newKey]}` != `${oldObj[newKey]}`) changeKeyList.push(newKey);
    }
    return changeKeyList;
}


/**
 * 查询孵化器基础数据
 * @param uscc 孵化器统一信用代码
 */
export async function selectFuHuaQiBaseData(uscc:string) {
    let dataBaseInfo = await findFuHuaQiByUSCC(uscc);

    let keyList = [ 
        "name", "operationName", "uscc", "virtualEnterpriseNum", "logonTime", "incubatedAcreage", "acreageTotal", "acreagePersonalUse", 
        "lv", "identificationTime","industry","institutionalNature","liaison","liaisonPhone","personInCharge","personInChargePhone",
        "personInChargeAdd","siteAcreage","leasePrice"];
    let data = {};

    keyList.forEach( key => {
        data[key] = dataBaseInfo[key];
    });

    return {data};
}