import {Schema} from 'mongoose';
import { baseDB } from '../../db/dbInit';

const financingSchema = new Schema({
    uscc:{type:String, index:true},//融资企业统一信用代码 冗余字段
    fuHuaQiUscc:{type:String, index:true},//孵化器统一信用代码 冗余字段
    name:String,//企业名称
    taskId:{type:String, index:true},//任务id
    logonAdd:String,//注册地址
    operatingAdd:String,//经营地址
    financingAmount:Number,//融资金额(万元)
    investmentInstitutionsName:String,//投资机构名称
    timeToObtainInvestment:Number,//获得投资时间
    fuHuaQiInvestment:{type:Boolean, default:false},//孵化器是否投资
    fuHuaQiInvestmentAmount:Number,//孵化器投资金额(万元)
    fuHuaQiInvestmentStyle:Number,//孵化器投资方式
    draftLock:{type:Boolean, default:false},//草稿锁，true为提交之后，false为草稿
    createTime:Number,//录入时间
});

var financingModel;
export function initModel(){
    financingModel = baseDB.model('financing', financingSchema);
    financingModel.selectOnceData = async function (paramater:object) {
        let selectInfo = await financingModel.findOne(paramater).exec();
        if (selectInfo) {
            if (!selectInfo.runSave) {
                selectInfo.runSave = selectInfo.save;
                selectInfo.save = save.bind(selectInfo)
            }
        }
        return selectInfo;
    }
}
export async function save(throwError=false) {
    if (!this.isModified()) return;
    await this.runSave({validateBeforeSave:false}).catch(err=>{
        console.log(err);
    });
}


/**
 * 通过taskId 获取此次任务添加的融资列表
 * @param taskId 
 * @returns 
 */
export async function findFinancingListByTaskId(taskId:string) {
    return await financingModel.find({taskId}).exec();
}

/**
 * 通过taskId和uscc获取融资信息你
 * @param taskId 
 * @returns 
 */
export async function findFinancingInfoByTaskIdAndSucc(taskId:string, uscc:string) {
    return await financingModel.selectOnceData({taskId, uscc}).exec();
}


/**
 * 修改特定taskId的锁为true
 * @param taskId 任务id
 */
export async function updateFinancingDraftLock(taskId:string) {
    return await financingModel.update({taskId}, {$set:{draftLock:true}}, {upsert:true});
}

/**
 * 创建新的融资数据
 * @param param 
 */
export async function createFinancing(fuHuaQiUscc:string, taskId:string, param) {
    let addInfo = Object.assign({fuHuaQiUscc, taskId, createTime:new Date().valueOf()}, param);
    await financingModel.create(addInfo);
}